from decimal import Decimal

from django import forms
from edc_constants.constants import NO, YES
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators import INVALID_ERROR
from edc_glucose.utils import validate_glucose_as_millimoles_per_liter

from ..models import GlucoseFbg


class GlucoseFbgFormValidator(CrfFormValidator):
    def clean(self):
        converted_value = None
        self.required_if(YES, field="fasting", field_required="fasting_duration_str")
        self.required_if(NO, field="fbg_performed", field_required="fbg_not_performed_reason")
        self.required_if(YES, field="fbg_performed", field_required="fbg_datetime")
        if self.cleaned_data.get("fbg_datetime") and self.cleaned_data.get(
            "fbg_datetime"
        ) < self.cleaned_data.get("report_datetime"):
            self.raise_validation_error(
                {"fbg_datetime": "Invalid. Must be on or after report date above"},
                INVALID_ERROR,
            )
        self.required_if(YES, field="fbg_performed", field_required="fbg_value")
        if self.cleaned_data.get("fbg_value") is not None:
            converted_value = validate_glucose_as_millimoles_per_liter(
                "fbg", self.cleaned_data
            )

        self.applicable_if(YES, field="fbg_performed", field_applicable="fbg_units")

        # repeat_fbg_date
        condition = converted_value and converted_value >= Decimal("7.0")
        self.required_if_true(condition, field_required="repeat_fbg_date")
        if self.cleaned_data.get("repeat_fbg_date") and self.cleaned_data.get("fbg_datetime"):
            diffdays = (
                self.cleaned_data.get("repeat_fbg_date")
                - self.cleaned_data.get("fbg_datetime").date()
            ).days
            if not (7 <= diffdays <= 10):
                self.raise_validation_error(
                    {
                        "repeat_fbg_date": (
                            f"Must be 7 to 10 days from date measured above. Got {diffdays}."
                        )
                    },
                    INVALID_ERROR,
                )


class GlucoseFbgForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = GlucoseFbgFormValidator

    class Meta:
        model = GlucoseFbg
        fields = "__all__"



from edc_constants.constants import NO, TBD, YES

from meta_consent.consents import consent_v1

from .subject_screening import SubjectScreening

options = dict(
    eligible_value_default=TBD,
    eligible_values_list=[YES, NO, TBD],
    is_eligible_value=YES,
)


class ScreeningPartOne(SubjectScreening):
    consent_definition = consent_v1

    def save(self, *args, **kwargs):
        if self.eligible_part_one == YES:
            self.continue_part_two = YES
        else:
            self.continue_part_two = NO
        super().save(*args, **kwargs)

    class Meta:
        proxy = True
        verbose_name = "Subject Screening: Part One"
        verbose_name_plural = "Subject Screening: Part One"


class ScreeningPartTwo(SubjectScreening):
    consent_definition = consent_v1

    class Meta:
        proxy = True
        verbose_name = "Subject Screening: Part Two"
        verbose_name_plural = "Subject Screening: Part Two"


class ScreeningPartThree(SubjectScreening):
    consent_definition = consent_v1

    class Meta:
        proxy = True
        verbose_name = "Subject Screening: Part Three"
        verbose_name_plural = "Subject Screening: Part Three"



from edc_pharmacy.models import Rx as BaseRx


class Rx(BaseRx):
    """A proxy model of edc_pharmacy.Rx.

    A model for the prescription.

    For autocomplete only.
    """

    def save(self, *args, **kwargs):
        raise NotImplementedError(
            "This proxy model may not be saved. Permissions should be view only"
        )

    class Meta:
        proxy = True



from edc_qareports.sql_generator import SqlViewGenerator


def get_view_definition() -> dict:
    subquery = """select distinct subject_identifier, site_id,  `first_value`, `second_value`, `third_value`,
datediff(`third_date`, `first_date`) as interval_days,
datediff(now(), `first_date`) as from_now_days,
`first_status`, `second_status`, `third_status`, sum(missed) as missed_count
from (
select subject_identifier,site_id,appt_datetime, `missed`,
FIRST_VALUE(appt_status) OVER w as `third_status`,
NTH_VALUE(appt_status, 2) OVER w as `second_status`,
NTH_VALUE(appt_status, 3) OVER w as `first_status`,
FIRST_VALUE(visit_code) OVER w as `third_value`,
NTH_VALUE(visit_code, 2) OVER w as `second_value`,
NTH_VALUE(visit_code, 3) OVER w as `first_value`,
FIRST_VALUE(appt_datetime) OVER w as `third_date`,
NTH_VALUE(appt_datetime, 2) OVER w as `second_date`,
NTH_VALUE(appt_datetime, 3) OVER w as `first_date`
from appointments
WINDOW w as (PARTITION BY subject_identifier order by appt_datetime desc ROWS UNBOUNDED PRECEDING)
) as B
where `second_value` is not null and `third_value` is not null
and `first_status`="New"
and `second_status`="New"
and `third_status` ="New"
group by subject_identifier, site_id, `first_value`, `second_value`, `third_value`,
datediff(`third_date`, `first_date`),
datediff(now(), `first_date`),
`first_status`, `second_status`, `third_status`
order by subject_identifier, site_id"""  # noqa

    with_stmt = """with appointments as (select subject_identifier, site_id, visit_code, visit_code_sequence,
appt_datetime, case when appt_timing="missed" then "New" else appt_status end as `appt_status`,
case when appt_timing="missed" then 1 else 0 end as `missed`
from edc_appointment_appointment
where visit_code_sequence=0 and appt_datetime<=now()
order by subject_identifier, appt_datetime)"""  # noqa

    sql_view = SqlViewGenerator(
        with_stmt=with_stmt,
        report_model="meta_reports.unattendedthreeinrow2",
        ordering=["subject_identifier", "site_id"],
    )

    return {
        "django.db.backends.mysql": sql_view.as_mysql(subquery),
        "django.db.backends.postgresql": sql_view.as_postgres(subquery),
        "django.db.backends.sqlite3": sql_view.as_sqlite(subquery),
    }



# Generated by Django 4.1.2 on 2022-11-30 16:31

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        (
            "meta_screening",
            "0057_alter_historicalscreeningpartone_calculated_egfr_value_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="eligibility_datetime",
            field=models.DateTimeField(
                help_text="Date and time eligibility was determined relative to report_datetime",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="eligible",
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="real_eligibility_datetime",
            field=models.DateTimeField(
                help_text="Date and time eligibility was determined relative to now", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="reasons_ineligible",
            field=models.TextField(
                max_length=150, null=True, verbose_name="Reason not eligible"
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="eligibility_datetime",
            field=models.DateTimeField(
                help_text="Date and time eligibility was determined relative to report_datetime",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="eligible",
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="real_eligibility_datetime",
            field=models.DateTimeField(
                help_text="Date and time eligibility was determined relative to now", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="reasons_ineligible",
            field=models.TextField(
                max_length=150, null=True, verbose_name="Reason not eligible"
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="eligibility_datetime",
            field=models.DateTimeField(
                help_text="Date and time eligibility was determined relative to report_datetime",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="eligible",
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="real_eligibility_datetime",
            field=models.DateTimeField(
                help_text="Date and time eligibility was determined relative to now", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="reasons_ineligible",
            field=models.TextField(
                max_length=150, null=True, verbose_name="Reason not eligible"
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="eligibility_datetime",
            field=models.DateTimeField(
                help_text="Date and time eligibility was determined relative to report_datetime",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="eligible",
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="real_eligibility_datetime",
            field=models.DateTimeField(
                help_text="Date and time eligibility was determined relative to now", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="reasons_ineligible",
            field=models.TextField(
                max_length=150, null=True, verbose_name="Reason not eligible"
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="eligibility_datetime",
            field=models.DateTimeField(
                help_text="Date and time eligibility was determined relative to report_datetime",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="eligible",
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="real_eligibility_datetime",
            field=models.DateTimeField(
                help_text="Date and time eligibility was determined relative to now", null=True
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="reasons_ineligible",
            field=models.TextField(
                max_length=150, null=True, verbose_name="Reason not eligible"
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
    ]



# Generated by Django 3.0.6 on 2020-05-16 22:50

import edc_model.validators.date
import edc_protocol.validators
import edc_utils.date
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0035_auto_20200517_0128"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalmissedvisit",
            name="contact_attempts_explained",
            field=models.TextField(
                blank=True,
                null=True,
                verbose_name="If less than 3 attempts, please explain",
            ),
        ),
        migrations.AddField(
            model_name="historicalmissedvisit",
            name="survival_status",
            field=models.CharField(
                choices=[
                    ("alive", "Alive"),
                    ("dead", "Deceased"),
                    ("unknown", "Unknown"),
                ],
                default="alive",
                max_length=25,
                verbose_name="Survival status",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="missedvisit",
            name="contact_attempts_explained",
            field=models.TextField(
                blank=True,
                null=True,
                verbose_name="If less than 3 attempts, please explain",
            ),
        ),
        migrations.AddField(
            model_name="missedvisit",
            name="survival_status",
            field=models.CharField(
                choices=[
                    ("alive", "Alive"),
                    ("dead", "Deceased"),
                    ("unknown", "Unknown"),
                ],
                default="alive",
                max_length=25,
                verbose_name="Survival status",
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="historicalmissedvisit",
            name="contact_last_date",
            field=models.DateField(
                blank=True,
                default=edc_utils.date.get_utcnow,
                null=True,
                validators=[
                    edc_model.validators.date.date_not_future,
                    edc_protocol.validators.date_not_before_study_start,
                ],
                verbose_name="Date of last telephone contact/attempt",
            ),
        ),
        migrations.AlterField(
            model_name="missedvisit",
            name="contact_last_date",
            field=models.DateField(
                blank=True,
                default=edc_utils.date.get_utcnow,
                null=True,
                validators=[
                    edc_model.validators.date.date_not_future,
                    edc_protocol.validators.date_not_before_study_start,
                ],
                verbose_name="Date of last telephone contact/attempt",
            ),
        ),
    ]



from reportlab.graphics.barcode.widgets import BarcodeCode128
from reportlab.graphics.shapes import Drawing

from .draw_label_for_subject_with_barcode import draw_label_for_subject_with_barcode


def draw_label_for_subject_with_code128(*args) -> Drawing:
    barcode_opts = dict(
        humanReadable=True,
        barHeight=30,
        barWidth=0.7,
        gap=1.7,
    )
    return draw_label_for_subject_with_barcode(BarcodeCode128, barcode_opts, *args)



# Generated by Django 2.2.6 on 2019-11-07 01:27

from django.db import migrations
from edc_reportable import MILLIMOLES_PER_LITER


def update_fbg(apps, schema_editor):
    subject_screening_model_cls = apps.get_model("meta_screening", "SubjectScreening")
    for obj in subject_screening_model_cls._default_manager.all():
        eligible = obj.eligible
        obj.converted_fasting_glucose = obj.fasting_glucose
        obj.fasting_glucose_units = MILLIMOLES_PER_LITER
        obj.save()
        obj.refresh_from_db()
        if obj.eligible != eligible:
            print(f"Eligibility has changed! See {obj.screening_identifier}")


class Migration(migrations.Migration):
    dependencies = [("meta_screening", "0011_auto_20191107_0342")]

    operations = [migrations.RunPython(update_fbg)]



import pandas as pd

from .get_screening_df import get_screening_df


def get_glucose_tested_only_df(df: pd.DataFrame | None = None):
    """ "Returns a DF of 5618 records"""
    df = pd.DataFrame() if not hasattr(df, "empty") else df
    if df.empty:
        df = get_screening_df()
    # condition where subject is eligible P1/P2 and has any type of glucose test
    cond_glu = (
        (df["fbg_value"].notna())
        | (df["ogtt_value"].notna())
        | (df["fbg2_value"].notna())
        | (df["ogtt2_value"].notna())
    )
    cond = (df["eligible_part_one"] == "Yes") & (df["eligible_part_two"] == "Yes") & cond_glu
    df = df[cond]
    return df



# Generated by Django 4.2.11 on 2024-04-04 14:18

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("edc_action_item", "0037_remove_actionitem_reference_model_and_more"),
        ("meta_lists", "0018_missedreferralreasons"),
        ("meta_subject", "0181_dmreferralfollowup_action_identifier_and_more"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="DmReferralFollowup",
            new_name="DmFollowup",
        ),
        migrations.RenameModel(
            old_name="HistoricalDmReferralFollowup",
            new_name="HistoricalDmFollowup",
        ),
        migrations.AlterModelOptions(
            name="dmfollowup",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "verbose_name": "Diabetes follow-up after referral",
                "verbose_name_plural": "Diabetes follow-up after referral",
            },
        ),
        migrations.AlterModelOptions(
            name="historicaldmfollowup",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Diabetes follow-up after referral",
                "verbose_name_plural": "historical Diabetes follow-up after referral",
            },
        ),
        migrations.RenameIndex(
            model_name="dmfollowup",
            new_name="meta_subjec_subject_dd09d5_idx",
            old_name="meta_subjec_subject_cfb7da_idx",
        ),
        migrations.RenameIndex(
            model_name="dmfollowup",
            new_name="meta_subjec_subject_1313a7_idx",
            old_name="meta_subjec_subject_257846_idx",
        ),
    ]



# Generated by Django 4.2.11 on 2024-04-03 19:45

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("edc_action_item", "0037_remove_actionitem_reference_model_and_more"),
        ("meta_subject", "0180_dmreferralfollowup_missed_referral_reasons_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="dmreferralfollowup",
            name="action_identifier",
            field=models.CharField(blank=True, max_length=50, null=True, unique=True),
        ),
        migrations.AddField(
            model_name="dmreferralfollowup",
            name="action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AddField(
            model_name="dmreferralfollowup",
            name="action_item_reason",
            field=models.TextField(editable=False, null=True),
        ),
        migrations.AddField(
            model_name="dmreferralfollowup",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="dmreferralfollowup",
            name="parent_action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AddField(
            model_name="dmreferralfollowup",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="dmreferralfollowup",
            name="related_action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AddField(
            model_name="historicaldmreferralfollowup",
            name="action_identifier",
            field=models.CharField(blank=True, db_index=True, max_length=50, null=True),
        ),
        migrations.AddField(
            model_name="historicaldmreferralfollowup",
            name="action_item",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AddField(
            model_name="historicaldmreferralfollowup",
            name="action_item_reason",
            field=models.TextField(editable=False, null=True),
        ),
        migrations.AddField(
            model_name="historicaldmreferralfollowup",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="historicaldmreferralfollowup",
            name="parent_action_item",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AddField(
            model_name="historicaldmreferralfollowup",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="historicaldmreferralfollowup",
            name="related_action_item",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
    ]



from django.apps import apps as django_apps
from django.contrib.sites.managers import CurrentSiteManager
from django.contrib.sites.models import Site
from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.db import models
from django_crypto_fields.fields.identity_field import IdentityField
from edc_action_item.models import ActionModelMixin
from edc_consent.field_mixins import ReviewFieldsMixin
from edc_constants.constants import ABNORMAL
from edc_identifier.managers import SubjectIdentifierManager
from edc_identifier.model_mixins import UniqueSubjectIdentifierModelMixin
from edc_model.models import BaseUuidModel
from edc_registration.models import RegisteredSubject
from edc_sites.model_mixins import SiteModelMixin
from edc_utils import get_utcnow

from ..action_items import RECONSENT_ACTION
from .model_mixins import SearchSlugModelMixin


class SubjectReconsent(
    UniqueSubjectIdentifierModelMixin,
    SiteModelMixin,
    ReviewFieldsMixin,
    SearchSlugModelMixin,
    ActionModelMixin,
    BaseUuidModel,
):
    """A model completed by the user that updates the consent
    for those originally consented by next of kin.
    """

    subject_identifier_cls = None

    subject_screening_model = "meta_screening.subjectscreening"

    subject_consent_model = "meta_consent.subjectconsentv1"

    tracking_identifier_prefix = "SR"

    action_name = RECONSENT_ACTION

    site = models.ForeignKey(Site, on_delete=models.PROTECT, null=True, editable=False)

    report_datetime = models.DateTimeField(default=get_utcnow)

    identity = IdentityField(
        verbose_name="Identity number",
        help_text=(
            "Provide the same identity number provided on the original "
            "consent complete by the next of kin."
        ),
    )

    on_site = CurrentSiteManager()

    objects = SubjectIdentifierManager()

    def __str__(self):
        return self.subject_identifier

    def save(self, *args, **kwargs):
        subject_screening = self.get_subject_screening()
        if subject_screening.mental_status != ABNORMAL:
            raise ValidationError("Reconsent is not required.")
        try:
            self.get_subject_consent(
                screening_identifier=subject_screening.screening_identifier
            )
        except ObjectDoesNotExist:
            raise ValidationError("Previous consent does not exist.")
        super().save(*args, **kwargs)

    def natural_key(self):
        return (self.subject_identifier, self.version)

    def get_subject_consent(self, screening_identifier=None):
        """Returns the first subject consent model instance."""
        if not screening_identifier:
            screening_identifier = self.get_subject_screening().screening_identifier
        model_cls = django_apps.get_model(self.subject_consent_model)
        return model_cls.objects.get(
            screening_identifier=screening_identifier,
            subject_identifier=self.subject_identifier,
            identity=self.identity,
        )

    def get_subject_screening(self):
        """Returns the subject screening model instance."""
        rs = RegisteredSubject.objects.get(subject_identifier=self.subject_identifier)
        model_cls = django_apps.get_model(self.subject_screening_model)
        return model_cls.objects.get(screening_identifier=rs.screening_identifier)

    @property
    def registration_unique_field(self):
        """Required for UpdatesOrCreatesRegistrationModelMixin."""
        return "subject_identifier"

    class Meta:
        verbose_name = "Subject re-consent"



from typing import Tuple

part_one_fields: Tuple[str, ...] = (
    "report_datetime",
    "screening_consent",
    "meta_phase_two",
    "selection_method",
    "hospital_identifier",
    "initials",
    "gender",
    "age_in_years",
    "ethnicity",
    "site",
    "hiv_pos",
    "art_six_months",
    "on_rx_stable",
    "vl_undetectable",
    "lives_nearby",
    "staying_nearby_12",
    "pregnant",
    "continue_part_two",
)


part_two_fields: Tuple[str, ...] = (
    "part_two_report_datetime",
    "congestive_heart_failure",
    "liver_disease",
    "alcoholism",
    "acute_metabolic_acidosis",
    "renal_function_condition",
    "tissue_hypoxia_condition",
    "acute_condition",
    "metformin_sensitivity",
    "has_dm",
    "on_dm_medication",
    "already_fasted",
    "agree_to_p3",
    "advised_to_fast",
    "appt_datetime",
    "contact_number",
)


part_three_vitals_fields: Tuple[str, ...] = (
    "height",
    "weight",
    "sys_blood_pressure_one",
    "dia_blood_pressure_one",
    "sys_blood_pressure_two",
    "dia_blood_pressure_two",
    "severe_htn",
)


part_three_fbg_fields: Tuple[str, ...] = (
    "fasting",
    "fasting_duration_str",
    "fbg_datetime",
    "fbg_value",
    "fbg_units",
)


part_three_repeat_fbg_fields: Tuple[str, ...] = (
    "fbg2_datetime",
    "fbg2_value",
    "fbg2_units",
)

part_three_ogtt_fields: Tuple[str, ...] = (
    "ogtt_base_datetime",
    "ogtt_datetime",
    "ogtt_value",
    "ogtt_units",
)

part_three_repeat_ogtt_fields: Tuple[str, ...] = (
    "ogtt2_base_datetime",
    "ogtt2_datetime",
    "ogtt2_value",
    "ogtt2_units",
)

part_three_glucose_fields: Tuple[str, ...] = (
    *part_three_fbg_fields,
    *part_three_ogtt_fields,
    *("repeat_glucose_opinion", "repeat_appt_datetime", "contact_number"),
)

part_three_pregnancy_fields: Tuple[str, ...] = (
    "urine_bhcg_performed",
    "urine_bhcg_value",
    "urine_bhcg_date",
)

part_three_other_fields: Tuple[str, ...] = (
    "creatinine_performed",
    "creatinine_value",
    "creatinine_units",
    "hba1c_performed",
    "hba1c_datetime",
    "hba1c_value",
)

part_three_creatinine_fields: Tuple[str, ...] = (
    "creatinine_performed",
    "creatinine_value",
    "creatinine_units",
)

part_three_hba1c_fields: Tuple[str, ...] = (
    "hba1c_performed",
    "hba1c_datetime",
    "hba1c_value",
)

part_three_comment_fields: Tuple[str, ...] = (
    "unsuitable_for_study",
    "reasons_unsuitable",
    "unsuitable_agreed",
)

calculated_fields: Tuple[str, ...] = (
    "sys_blood_pressure_avg",
    "dia_blood_pressure_avg",
    "calculated_bmi_value",
    "converted_fbg_value",
    "converted_ogtt_value",
    "converted_fbg2_value",
    "converted_ogtt2_value",
    "converted_creatinine_value",
    "calculated_egfr_value",
    "inclusion_a",
    "inclusion_b",
    "inclusion_c",
    "inclusion_d",
)


part_three_labs: Tuple[str, ...] = (
    "haemoglobin_value",
    "wbc_value",
    "ast_value",
    "alt_value",
    "alp_value",
    "ggt_value",
    "albumin_value",
)


part_three_fields: Tuple[str, ...] = (
    "part_three_report_datetime",
    *part_three_vitals_fields,
    *part_three_pregnancy_fields,
    *part_three_fbg_fields,
    *part_three_ogtt_fields,
    *part_three_other_fields,
    *part_three_comment_fields,
)



from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist
from django.db.models.signals import post_save, pre_save
from django.dispatch import receiver
from edc_consent.utils import get_consent_model_cls
from edc_pharmacy.exceptions import PrescriptionAlreadyExists
from edc_pharmacy.prescribe import create_prescription
from edc_visit_schedule.constants import DAY1

from meta_edc.meta_version import get_meta_version
from meta_pharmacy.constants import METFORMIN
from meta_visit_schedule.constants import SCHEDULE_PREGNANCY

from . import Glucose, GlucoseFbg
from .delivery import Delivery
from .study_medication import StudyMedication
from .subject_visit import SubjectVisit


@receiver(
    pre_save,
    weak=False,
    sender=StudyMedication,
    dispatch_uid="study_medication_on_pre_save",
)
def study_medication_on_pre_save(sender, instance, raw, **kwargs):
    """Create a prescription if one does not exist

    All refills are created against the prescription

    Note: this should not be necessary if consented after meta_consent.signal
          was updated to call `create_prescription`.
    """
    if not raw and instance.subject_visit.visit_code == DAY1:
        subject_consent = get_consent_model_cls().objects.get(
            subject_identifier=instance.subject_visit.subject_identifier,
        )
        try:
            create_prescription(
                subject_identifier=subject_consent.subject_identifier,
                report_datetime=subject_consent.consent_datetime,
                randomizer_name=get_meta_version(),
                medications=[METFORMIN],
                site=subject_consent.site,
            )
        except PrescriptionAlreadyExists:
            pass


@receiver(
    post_save,
    weak=False,
    sender=Delivery,
    dispatch_uid="update_pregnancy_notification_on_delivery_post_save",
)
def update_pregnancy_notification_on_delivery_post_save(sender, instance, raw, **kwargs):
    """Updates PregnancyNotification model instance when the delivery form is submitted.

    Sets delivered=True and delivery_datetime
    """
    if not raw:
        model_cls = django_apps.get_model("meta_prn.pregnancynotification")
        model_cls.objects.filter(
            subject_identifier=instance.subject_visit.subject_identifier,
            delivered=False,
            delivery_datetime=None,
        ).update(delivered=True, delivery_datetime=instance.delivery_datetime)


@receiver(
    post_save,
    weak=False,
    sender=Delivery,
    dispatch_uid="update_schedule_on_delivery_post_save",
)
def update_schedule_on_delivery_post_save(sender, instance, raw, **kwargs):
    """Takes a participant off the SCHEDULE_PREGNANCY schedule when
    delivery model is submitted.

    - gets last report_datetime of the subject visit for
      SCHEDULE_PREGNANCY
    - automatically submit offschedulepregnancy model
    """
    if not raw:
        offschedule_model_cls = django_apps.get_model("meta_prn.offschedulepregnancy")
        try:
            offschedule_model_cls.objects.get(subject_identifier=instance.subject_identifier)
        except ObjectDoesNotExist:
            last_subject_visit = (
                SubjectVisit.objects.filter(
                    subject_identifier=instance.subject_identifier,
                    schedule_name=SCHEDULE_PREGNANCY,
                )
                .order_by("report_datetime")
                .last()
            )
            delivery = Delivery.objects.get(
                subject_visit__subject_identifier=instance.subject_identifier,
            )
            offschedule_datetime = (
                delivery.delivery_datetime
                if delivery.delivery_datetime > last_subject_visit.report_datetime
                else last_subject_visit.report_datetime
            )
            offschedule_model_cls.objects.create(
                subject_identifier=instance.subject_identifier,
                offschedule_datetime=offschedule_datetime,
            )


@receiver(
    post_save,
    weak=False,
    dispatch_uid="update_glucose_endpoints_for_subject_on_post_save",
)
def update_glucose_endpoints_for_subject_on_post_save(
    sender, instance, raw, update_fields, **kwargs
):
    if not raw and not update_fields and sender in [Glucose, GlucoseFbg]:
        from meta_analytics.dataframes import GlucoseEndpointsByDate

        cls = GlucoseEndpointsByDate(
            subject_identifiers=[instance.subject_visit.subject_identifier]
        )
        cls.run()
        cls.to_model()



# Generated by Django 5.0.7 on 2024-08-13 12:17

import django_db_views.migration_functions
import django_db_views.operations
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0013_auto_20240813_1516"),
    ]

    operations = [
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as `id`, now() as `created`, 'meta_reports.unattendedtwoinrow' as report_model from (\n        select subject_identifier, site_id, appt_datetime, `first_value`, `second_value`,\n            datediff(`second_date`, `first_date`) as `interval_days`,\n            datediff(now(), `first_date`) as `from_now_days`\n            from (\n                select subject_identifier ,site_id, appt_datetime,\n                FIRST_VALUE(`visit_code`) OVER w as `first_value`,\n                NTH_VALUE(`visit_code`, 2) OVER w as `second_value`,\n                NTH_VALUE(`visit_code`, 3) OVER w as `third_value`,\n                FIRST_VALUE(`appt_datetime`) OVER w as `first_date`,\n                NTH_VALUE(`appt_datetime`, 2) OVER w as `second_date`\n                from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n                and appt_datetime <= now()\n                WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n            ) as A1\n        where `second_value` is not null and `third_value` is null\n    ) as A2\n    order by site_id, `from_now_days` desc",
                "unattended_two_in_row_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "", "unattended_two_in_row_view", engine="django.db.backends.mysql"
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, gen_random_uuid() as id, now() as created, 'meta_reports.unattendedtwoinrow' as report_model \n    from (\n        select subject_identifier, site_id, appt_datetime, first_value, second_value,\n            EXTRACT(DAY FROM second_date - first_date) as interval_days,\n            EXTRACT(DAY FROM now() - first_date) as from_now_days\n            from (\n                select subject_identifier, site_id, appt_datetime,\n                FIRST_VALUE(visit_code) OVER w as first_value,\n                NTH_VALUE(visit_code, 2) OVER w as second_value,\n                NTH_VALUE(visit_code, 3) OVER w as third_value,\n                FIRST_VALUE(appt_datetime) OVER w as first_date,\n                NTH_VALUE(appt_datetime, 2) OVER w as second_date\n                from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n                and appt_datetime <= now()\n                WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n            ) as A1\n        where second_value is not null and third_value is null\n    ) as A2\n    order by site_id, from_now_days desc",
                "unattended_two_in_row_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "", "unattended_two_in_row_view", engine="django.db.backends.postgresql"
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "SELECT *, lower(\n    hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||\n    substr(hex( randomblob(2)), 2) || '-' ||\n    substr('AB89', 1 + (abs(random()) % 4) , 1)  ||\n    substr(hex(randomblob(2)), 2) || '-' ||\n    hex(randomblob(6))\n  ) as id, datetime() as created, 'meta_reports.unattendedtwoinrow' as report_model from (\n        select subject_identifier, site_id, appt_datetime, first_value, second_value,\n            CAST(JulianDay(second_date) - JulianDay(first_date) AS Integer) as interval_days,\n            CAST(JulianDay(datetime()) - JulianDay(first_date) AS Integer) as from_now_days\n            from (\n                select subject_identifier ,site_id, appt_datetime,\n                FIRST_VALUE(visit_code) OVER w as first_value,\n                NTH_VALUE(visit_code, 2) OVER w as second_value,\n                NTH_VALUE(visit_code, 3) OVER w as third_value,\n                FIRST_VALUE(appt_datetime) OVER w as first_date,\n                NTH_VALUE(appt_datetime, 2) OVER w as second_date\n                from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n                and appt_datetime <= datetime()\n                WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n            ) as A1\n        where second_value is not null and third_value is null\n    ) as A2\n    order by site_id, from_now_days desc",
                "unattended_two_in_row_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "", "unattended_two_in_row_view", engine="django.db.backends.sqlite3"
            ),
            atomic=False,
        ),
    ]



from dateutil.relativedelta import relativedelta
from django.core.exceptions import ObjectDoesNotExist
from django.test import TestCase
from edc_appointment.models import Appointment
from edc_constants.constants import FEMALE, NO, YES
from edc_utils import get_utcnow
from edc_visit_schedule.constants import MONTH1
from model_bakery.baker import make_recipe

from meta_prn.models import OffSchedule
from meta_prn.models.pregnancy_notification import PregnancyNotificationError
from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin
from meta_visit_schedule.constants import SCHEDULE, SCHEDULE_PREGNANCY


class TestPregnancyNotification(MetaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.subject_visit = self.get_subject_visit(gender=FEMALE)

    def test_pregnancy_notification_updates_urine_pregnancy(self):
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        subject_visit = self.get_next_subject_visit(subject_visit)
        self.assertEqual(subject_visit.visit_code, MONTH1)
        urine_pregnancy = make_recipe(
            "meta_subject.urinepregnancy",
            subject_visit=subject_visit,
            report_datetime=get_utcnow(),
            assay_date=get_utcnow().date(),
        )

        self.assertFalse(urine_pregnancy.notified)
        self.assertIsNotNone(urine_pregnancy.assay_date)
        make_recipe(
            "meta_prn.pregnancynotification",
            subject_identifier=subject_visit.subject_identifier,
            report_datetime=get_utcnow(),
        )
        urine_pregnancy.refresh_from_db()
        self.assertTrue(urine_pregnancy.notified)
        self.assertIsNotNone(urine_pregnancy.notified_datetime)

    def test_pregnancy_notification_unconfirmed_ok(self):
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        subject_visit = self.get_next_subject_visit(subject_visit)
        self.assertEqual(subject_visit.visit_code, MONTH1)
        make_recipe(
            "meta_prn.pregnancynotification",
            subject_identifier=subject_visit.subject_identifier,
            report_datetime=get_utcnow(),
            bhcg_confirmed=NO,
            unconfirmed_details="blah blah",
        )

    def test_pregnancy_notification_confirmed_raises(self):
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        subject_visit = self.get_next_subject_visit(subject_visit)
        self.assertEqual(subject_visit.visit_code, MONTH1)
        try:
            make_recipe(
                "meta_prn.pregnancynotification",
                subject_identifier=subject_visit.subject_identifier,
                report_datetime=get_utcnow(),
                bhcg_confirmed=YES,
                unconfirmed_details="blah blah",
            )
        except PregnancyNotificationError:
            pass
        else:
            self.fail("PregnancyNotificationError unexpectedly NOT raised")

    def test_pregnancy_notification_takes_off_schedule(self):
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        subject_visit = self.get_next_subject_visit(subject_visit)
        self.assertEqual(subject_visit.visit_code, MONTH1)
        make_recipe(
            "meta_prn.pregnancynotification",
            subject_identifier=subject_visit.subject_identifier,
            report_datetime=subject_visit.report_datetime,
            bhcg_confirmed=NO,
            unconfirmed_details="blah blah",
            edd=subject_visit.report_datetime + relativedelta(months=6),
        )

        try:
            OffSchedule.objects.get(subject_identifier=subject_visit.subject_identifier)
        except ObjectDoesNotExist:
            self.fail("OffSchedule unexpectedly does not exist")

        self.assertEqual(Appointment.objects.filter(schedule_name=SCHEDULE).count(), 3)
        self.assertEqual(
            Appointment.objects.filter(schedule_name=SCHEDULE_PREGNANCY).count(), 1
        )



# Generated by Django 2.2.7 on 2020-01-18 21:13

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0012_auto_20200118_2334"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalmalariatest",
            name="diagnostic_type",
            field=models.CharField(
                choices=[
                    ("rapid_test", "Rapid test"),
                    ("microscopy", "Microscopy"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="Diagnostic test used",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmalariatest",
            name="performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Was the malaria test performed?",
            ),
        ),
        migrations.AlterField(
            model_name="malariatest",
            name="diagnostic_type",
            field=models.CharField(
                choices=[
                    ("rapid_test", "Rapid test"),
                    ("microscopy", "Microscopy"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="Diagnostic test used",
            ),
        ),
        migrations.AlterField(
            model_name="malariatest",
            name="performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Was the malaria test performed?",
            ),
        ),
    ]



# Generated by Django 5.0.1 on 2024-02-14 03:17

import uuid

import _socket
import django.core.validators
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_crf.model_mixins.crf_status_model_mixin
import edc_model.validators.date
import edc_protocol.validators
import edc_utils.date
import edc_visit_tracking.managers
import simple_history.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0171_alter_glucose_endpoint_today_and_more"),
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.RenameModel(
            old_name="historicalbloodresultsglu",
            new_name="historicalglucosefbg",
        ),
        migrations.RenameModel(
            old_name="bloodresultsglu",
            new_name="glucosefbg",
        ),
        migrations.AlterModelOptions(
            name="glucosefbg",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "verbose_name": "Glucose (FBG)",
                "verbose_name_plural": "Glucose (FBG)",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalglucosefbg",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Glucose (FBG)",
                "verbose_name_plural": "historical Glucose (FBG)",
            },
        ),
        migrations.RenameIndex(
            model_name="glucosefbg",
            new_name="meta_subjec_subject_e585cf_idx",
            old_name="meta_subjec_subject_199c7b_idx",
        ),
        migrations.RenameIndex(
            model_name="glucosefbg",
            new_name="meta_subjec_subject_b1d1d3_idx",
            old_name="meta_subjec_subject_93fda3_idx",
        ),
    ]



# Generated by Django 5.0.1 on 2024-02-13 01:56

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "meta_subject",
            "0167_rename_healthcare_workers_other_dmreferralfollowup_other_healthcare_workers_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="dmreferralfollowup",
            name="last_missed_pill",
            field=models.CharField(
                blank=True,
                choices=[
                    ("today", "today"),
                    ("yesterday", "yesterday"),
                    ("earlier_this_week", "earlier this week"),
                    ("last_week", "last week"),
                    ("lt_month_ago", "less than a month ago"),
                    ("gt_month_ago", "more than a month ago"),
                    ("NEVER", "have never missed taking my study pills"),
                ],
                max_length=25,
                null=True,
                verbose_name="When was the last time you missed your study pill?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldmreferralfollowup",
            name="last_missed_pill",
            field=models.CharField(
                blank=True,
                choices=[
                    ("today", "today"),
                    ("yesterday", "yesterday"),
                    ("earlier_this_week", "earlier this week"),
                    ("last_week", "last week"),
                    ("lt_month_ago", "less than a month ago"),
                    ("gt_month_ago", "more than a month ago"),
                    ("NEVER", "have never missed taking my study pills"),
                ],
                max_length=25,
                null=True,
                verbose_name="When was the last time you missed your study pill?",
            ),
        ),
    ]



# Generated by Django 5.1.2 on 2024-11-18 02:18

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0213_rename_bloodresultslipid_bloodresultslipids_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalstudymedication",
            name="stock_codes",
            field=models.CharField(
                blank=True,
                help_text="Enter the medication bottle barcode or bacodes seperated by comma, no spaces",
                max_length=30,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        regex="^([a-zA-Z0-9]{6})(,[a-zA-Z0-9]{6})*$"
                    )
                ],
            ),
        ),
        migrations.AddField(
            model_name="studymedication",
            name="stock_codes",
            field=models.CharField(
                blank=True,
                help_text="Enter the medication bottle barcode or bacodes seperated by comma, no spaces",
                max_length=30,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        regex="^([a-zA-Z0-9]{6})(,[a-zA-Z0-9]{6})*$"
                    )
                ],
            ),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-14 22:32

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0007_auto_20191107_2249"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="haemoglobin_units",
            field=models.CharField(
                blank=True,
                choices=[("g/dL", "g/dL")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="hct",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=6,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(999.0),
                ],
                verbose_name="Hematocrit",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="hct_units",
            field=models.CharField(
                blank=True,
                choices=[("%", "%")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="platelets_units",
            field=models.CharField(
                blank=True,
                choices=[("10^9/L", "10^9/L"), ("cells/mm^3", "cells/mm<sup>3</sup>")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="rbc_units",
            field=models.CharField(
                blank=True,
                choices=[("10^9/L", "10^9/L"), ("cells/mm^3", "cells/mm^3")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsfbc",
            name="wbc_units",
            field=models.CharField(
                blank=True,
                choices=[("10^9/L", "10^9/L"), ("cells/mm^3", "cells/mm<sup>3</sup>")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsglu",
            name="glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultshba1c",
            name="hba1c_units",
            field=models.CharField(
                blank=True,
                choices=[("%", "%")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="albumin_units",
            field=models.CharField(
                blank=True,
                choices=[("g/dL", "g/dL"), ("g/L", "g/L")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="alp_units",
            field=models.CharField(
                blank=True,
                choices=[("IU/L", "U/L")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="alt_units",
            field=models.CharField(
                blank=True,
                choices=[("IU/L", "U/L")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="amylase_units",
            field=models.CharField(
                blank=True,
                choices=[("IU/L", "U/L")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="ast_units",
            field=models.CharField(
                blank=True,
                choices=[("IU/L", "U/L")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="ggt_units",
            field=models.CharField(
                blank=True,
                choices=[("IU/L", "U/L")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="creatinine",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Creatinine <u>level</u>",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="creatinine_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("umol/L", "μmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (creatinine)",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="urea_units",
            field=models.CharField(
                blank=True,
                choices=[("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="uric_acid",
            field=models.DecimalField(
                blank=True,
                decimal_places=4,
                max_digits=10,
                null=True,
                verbose_name="Uric Acid",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsrft",
            name="uric_acid_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="haemoglobin_units",
            field=models.CharField(
                blank=True,
                choices=[("g/dL", "g/dL")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="hct",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=6,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(999.0),
                ],
                verbose_name="Hematocrit",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="hct_units",
            field=models.CharField(
                blank=True,
                choices=[("%", "%")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="platelets_units",
            field=models.CharField(
                blank=True,
                choices=[("10^9/L", "10^9/L"), ("cells/mm^3", "cells/mm<sup>3</sup>")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="rbc_units",
            field=models.CharField(
                blank=True,
                choices=[("10^9/L", "10^9/L"), ("cells/mm^3", "cells/mm^3")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="wbc_units",
            field=models.CharField(
                blank=True,
                choices=[("10^9/L", "10^9/L"), ("cells/mm^3", "cells/mm<sup>3</sup>")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsglu",
            name="glucose_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultshba1c",
            name="hba1c_units",
            field=models.CharField(
                blank=True,
                choices=[("%", "%")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="albumin_units",
            field=models.CharField(
                blank=True,
                choices=[("g/dL", "g/dL"), ("g/L", "g/L")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="alp_units",
            field=models.CharField(
                blank=True,
                choices=[("IU/L", "U/L")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="alt_units",
            field=models.CharField(
                blank=True,
                choices=[("IU/L", "U/L")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="amylase_units",
            field=models.CharField(
                blank=True,
                choices=[("IU/L", "U/L")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="ast_units",
            field=models.CharField(
                blank=True,
                choices=[("IU/L", "U/L")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="ggt_units",
            field=models.CharField(
                blank=True,
                choices=[("IU/L", "U/L")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="creatinine",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                verbose_name="Creatinine <u>level</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="creatinine_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("umol/L", "μmol/L")],
                max_length=15,
                null=True,
                verbose_name="Units (creatinine)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="urea_units",
            field=models.CharField(
                blank=True,
                choices=[("mmol/L", "mmol/L")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="uric_acid",
            field=models.DecimalField(
                blank=True,
                decimal_places=4,
                max_digits=10,
                null=True,
                verbose_name="Uric Acid",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="uric_acid_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL")],
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
    ]



# Generated by Django 3.2.11 on 2022-04-12 18:51

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_consent", "0007_auto_20220128_1719"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalsubjectconsent",
            name="study_medication_name",
            field=models.CharField(
                default="metformin",
                editable=False,
                help_text="Used by `create_prescription` in the signal",
                max_length=25,
            ),
        ),
        migrations.AddField(
            model_name="subjectconsent",
            name="study_medication_name",
            field=models.CharField(
                default="metformin",
                editable=False,
                help_text="Used by `create_prescription` in the signal",
                max_length=25,
            ),
        ),
    ]



from django.db import models
from edc_action_item.models import ActionModelMixin
from edc_constants.choices import YES_NO, YES_NO_UNSURE
from edc_constants.constants import YES
from edc_identifier.model_mixins import NonUniqueSubjectIdentifierFieldMixin
from edc_model.models import BaseUuidModel
from edc_sites.model_mixins import SiteModelMixin
from edc_utils import get_utcnow

from meta_subject.models import UrinePregnancy

from ..constants import PREGNANCY_NOTIFICATION_ACTION


class PregnancyNotificationError(Exception):
    pass


class PregnancyNotification(
    SiteModelMixin,
    ActionModelMixin,
    NonUniqueSubjectIdentifierFieldMixin,
    BaseUuidModel,
):
    action_name = PREGNANCY_NOTIFICATION_ACTION

    tracking_identifier_prefix = "PN"

    report_datetime = models.DateTimeField(
        verbose_name="Report Date and Time", default=get_utcnow
    )

    bhcg_confirmed = models.CharField(
        verbose_name="Has the pregnancy been confirmed by urine βHCG?",
        max_length=5,
        choices=YES_NO,
        help_text="If YES, the UPT result must already be entered in the EDC.",
    )

    bhcg_date = models.DateField(
        verbose_name="βHCG result date",
        null=True,
        blank=True,
        help_text="Will be validated against a UPT result in the EDC.",
    )

    unconfirmed_details = models.TextField(
        verbose_name="If no, please provide details",
        null=True,
        blank=True,
    )

    edd = models.DateField(
        verbose_name="Estimated date of delivery",
    )

    # TODO: remove this question
    may_contact = models.CharField(
        verbose_name=(
            "Has the participant agreed to be contacted to provide "
            "information on the delivery and birth outcomes?"
        ),
        max_length=15,
        choices=YES_NO_UNSURE,
        help_text=(
            "If YES/UNSURE, a visit will be scheduled on the EDD. "
            "If NO, the participant will be taken off study now"
        ),
        default=YES,
    )

    delivered = models.BooleanField(
        default=False, editable=False, help_text="Auto updated from Delivery"
    )

    delivery_datetime = models.DateTimeField(
        verbose_name="Date and time of delivery :",
        null=True,
        editable=False,
        help_text="Auto updated from Delivery",
    )

    def save(self, *args, **kwargs):
        if (
            not self.id
            and self.bhcg_confirmed == YES
            and not UrinePregnancy.objects.filter(
                subject_visit__subject_identifier=self.subject_identifier,
                notified=False,
                assay_date__lte=self.report_datetime.date(),
            ).exists()
        ):
            raise PregnancyNotificationError(
                "Invalid. A positive Urine βhCG cannot be found. "
                "Perhaps catch this in the form."
            )
        super().save(*args, **kwargs)

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Pregnancy Notification"
        verbose_name_plural = "Pregnancy Notifications"
        unique_together = ["subject_identifier", "edd"]



from .constants import (
    CASE_EOS,
    CASE_FBG_ONLY,
    CASE_FBGS_WITH_FIRST_OGTT,
    CASE_FBGS_WITH_SECOND_OGTT,
    CASE_OGTT,
    endpoint_cases,
    endpoint_columns,
)
from .get_eos_df import get_eos_df
from .get_last_imp_visits_df import get_last_imp_visits_df
from .glucose_endpoints import EndpointByDate, GlucoseEndpointsByDate
from .screening import get_glucose_tested_only_df, get_screening_df
from .utils import (
    get_empty_endpoint_df,
    get_test_string,
    get_unique_subject_identifiers,
    get_unique_visit_codes,
)



# Generated by Django 5.0.8 on 2024-08-21 22:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0027_auto_20240822_0140"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="glucosesummary",
            options={
                "default_permissions": ("view", "export", "viewallsites"),
                "managed": False,
                "verbose_name": "Glucose Summary",
                "verbose_name_plural": "Glucose Summary",
            },
        ),
    ]



import numpy as np
import pandas as pd
from edc_constants.constants import YES

from ..constants import endpoint_cases


class EndpointTdeltaError(Exception):
    pass


class InvalidCaseList(Exception):
    pass


class EndpointByDate:
    """Given all timepoints for a subject, flag the first timepoint
    where the protocol endpoint is reached.

    IMPORTANT: Remove case one before passing to this class
        * case 1. any OGTT >= 11.1

    Evaluation is done in order

    Order of protocol endpoint evaluation:
      * case 2. FBG >= 7 x 2, first OGTT<11.1
      * case 3.  FBG >= 7 x 2, second OGTT<11.1

    Additional criteria considered:
      1. any threshhold FBG must be taken while fasted (fasting=YES)
      2. threshhold FBG readings must be consecutive (no
         readings below threshold in the sequence regardless
         of fasting)
      3. at least 7 days between threshhold FBG readings.
      4. at least one of the two threshold FBG readings must be taken
         with an OGTT at the same timepoint.

    Note:
        case 4 is not a protocol endpoint. It considers only FBG and fasting.
        It looks for two consecutive fasted threshold FBG readings.
    """

    valid_case_list = [2, 3, 4]

    def __init__(
        self,
        subject_df: pd.DataFrame = None,
        fbg_threshhold: float = None,
        ogtt_threshhold: float = None,
        case_list: list[int] | None = None,
    ):
        self.row = None
        self.index = None
        self.subject_df = subject_df[subject_df["fbg_value"].notna()]
        self.subject_df = self.subject_df.reset_index(drop=True)
        self.fbg_threshhold = fbg_threshhold
        self.ogtt_threshhold = ogtt_threshhold
        self.case_list = case_list or [2, 3]
        if [x for x in self.case_list if x not in self.valid_case_list]:
            raise InvalidCaseList(f"Expected any of {self.valid_case_list}. Got {case_list}.")
        self.endpoint_cases = {k: v for k, v in endpoint_cases.items() if k in self.case_list}
        self.evaluate()

    def evaluate(self):
        for index, _ in self.subject_df.iterrows():
            if 2 in self.case_list and self.case_two(index):
                break
            elif 3 in self.case_list and self.case_three(index):
                break
            elif 4 in self.case_list and self.case_four(index):
                break

    def endpoint_reached(self, index: int, case: int, next_is_endpoint: bool | None = None):
        """Update the subject_df"""
        fbg_datetime = (
            self.get_next("fbg_datetime", index)
            if next_is_endpoint
            else self.get("fbg_datetime", index)
        )
        self.subject_df.loc[self.subject_df["fbg_datetime"] == fbg_datetime, "endpoint"] = 1
        self.subject_df["interval_in_days"] = np.nan
        try:
            self.subject_df.loc[
                self.subject_df["fbg_datetime"] == fbg_datetime, "interval_in_days"
            ] = self.sequential_assessments_in_days(index)
        except EndpointTdeltaError:
            pass
        self.subject_df["interval_in_days"] = pd.to_numeric(
            self.subject_df["interval_in_days"]
        )
        self.subject_df.loc[
            self.subject_df["fbg_datetime"] == fbg_datetime, "endpoint_type"
        ] = case
        self.subject_df.loc[
            self.subject_df["fbg_datetime"] == fbg_datetime, "endpoint_label"
        ] = self.endpoint_cases[case]

    def case_two(self, index: int):
        """FBG >= 7 x 2, first OGTT<11.1.

        First FBG must be done with corresponding OGTT.
        """
        reached = (
            self.get_next("fbg_datetime", index)
            and self.get("fbg_value", index)
            and self.get("ogtt_value", index)
            and self.get("fasting", index)
            and self.get_next("fbg_value", index)
            and self.get_next("fasting", index)
            and self.get("fbg_value", index) >= self.fbg_threshhold
            and self.get("ogtt_value", index) < self.ogtt_threshhold
            and self.get("fasting", index) == YES
            and self.get_next("fbg_value", index) >= self.fbg_threshhold
            and self.get_next("fasting", index) == YES
            and (self.get_next("fbg_datetime", index) - self.get("fbg_datetime", index)).days
            >= 7
        )
        if reached:
            self.endpoint_reached(index, case=2, next_is_endpoint=True)
        return reached

    def case_three(self, index: int):
        """FBG >= 7 x 2, second OGTT<11.1.

        Second FBG must be done with corresponding OGTT.
        """
        reached = (
            self.get_next("fbg_datetime", index)
            and self.get("fbg_value", index)
            and self.get("fasting", index)
            and self.get_next("fbg_value", index)
            and self.get_next("ogtt_value", index)
            and self.get_next("fasting", index)
            and self.get("fbg_value", index) >= self.fbg_threshhold
            and self.get("fasting", index) == YES
            and self.get_next("fbg_value", index) >= self.fbg_threshhold
            and self.get_next("ogtt_value", index) < self.ogtt_threshhold
            and self.get_next("fasting", index) == YES
            and (self.get_next("fbg_datetime", index) - self.get("fbg_datetime", index)).days
            >= 7
        )
        if reached:
            self.endpoint_reached(index, case=3, next_is_endpoint=True)
        return reached

    def case_four(self, index: int):
        """FBG >= 7 x 2, OGTT not considered

        This is not a protocol endpoint.
        """
        reached = (
            self.get("fbg_value", index)
            and self.get("fbg_datetime", index)
            and self.get("fasting", index)
            and self.get_next("fbg_value", index)
            and self.get_next("ogtt_value", index)
            and self.get_next("fbg_datetime", index)
            and self.get_next("fasting", index)
            and self.get("fbg_value", index) >= self.fbg_threshhold
            and self.get("fasting", index) == YES
            and self.get_next("fbg_value", index) >= self.fbg_threshhold
            and self.get_next("fasting", index) == YES
            and (self.get_next("fbg_datetime", index) - self.get("fbg_datetime", index)).days
            >= 7
        )
        if reached:
            self.endpoint_reached(index, case=4, next_is_endpoint=True)
        return reached

    def sequential_assessments_in_days(self, index) -> int:
        if not self.get_next("fbg_value", index):
            raise EndpointTdeltaError
        return (self.get_next("fbg_datetime", index) - self.get("visit_datetime", index)).days

    def get(self, col: str, index: int) -> float | None:
        try:
            next_value = self.subject_df.iloc[index : index + 1][col].item()
        except ValueError:
            next_value = None
        return next_value

    def get_next(self, col: str, index: int) -> float | None:
        return self.get(col, index + 1)



# Generated by Django 5.0 on 2024-01-09 23:21

import edc_model.validators.date
import edc_protocol.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "edc_action_item",
            "0035_alter_actionitem_options_alter_actiontype_options_and_more",
        ),
        ("edc_adverse_event", "0014_alter_aeactionclassification_options_and_more"),
        ("meta_ae", "0019_alter_aefollowup_managers_alter_aeinitial_managers_and_more"),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="aesusar",
            options={
                "verbose_name": "AE SUSAR Report",
                "verbose_name_plural": "AE SUSAR Reports",
            },
        ),
        migrations.AlterModelOptions(
            name="aetmg",
            options={
                "verbose_name": "AE TMG Report",
                "verbose_name_plural": "AE TMG Reports",
            },
        ),
        migrations.AlterModelOptions(
            name="deathreport",
            options={
                "verbose_name": "Death Report",
                "verbose_name_plural": "Death Reports",
            },
        ),
        migrations.RemoveIndex(
            model_name="aeinitial",
            name="meta_ae_aei_subject_9fefa3_idx",
        ),
        migrations.AddField(
            model_name="aefollowup",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="aefollowup",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="aeinitial",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="aeinitial",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="aelocalreview",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="aelocalreview",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="aesponsorreview",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="aesponsorreview",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="aesusar",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="aesusar",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="aetmg",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="aetmg",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="deathreport",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="deathreport",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="deathreporttmg",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="deathreporttmg",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalaefollowup",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalaefollowup",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalaeinitial",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalaeinitial",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalaesusar",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalaesusar",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalaetmg",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalaetmg",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicaldeathreport",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicaldeathreport",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicaldeathreporttmg",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicaldeathreporttmg",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicaldeathreporttmgsecond",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicaldeathreporttmgsecond",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="deathreport",
            name="death_date",
            field=models.DateField(
                null=True,
                validators=[
                    edc_protocol.validators.date_not_before_study_start,
                    edc_model.validators.date.date_not_future,
                ],
                verbose_name="Date of Death",
            ),
        ),
        migrations.AlterField(
            model_name="deathreport",
            name="death_datetime",
            field=models.DateTimeField(
                null=True,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Date and Time of Death",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldeathreport",
            name="death_date",
            field=models.DateField(
                null=True,
                validators=[
                    edc_protocol.validators.date_not_before_study_start,
                    edc_model.validators.date.date_not_future,
                ],
                verbose_name="Date of Death",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldeathreport",
            name="death_datetime",
            field=models.DateTimeField(
                null=True,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Date and Time of Death",
            ),
        ),
        migrations.AddIndex(
            model_name="aefollowup",
            index=models.Index(
                fields=["subject_identifier"], name="meta_ae_aef_subject_054161_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="aefollowup",
            index=models.Index(
                fields=[
                    "action_identifier",
                    "action_item",
                    "related_action_item",
                    "parent_action_item",
                ],
                name="meta_ae_aef_action__b2479a_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="aeinitial",
            index=models.Index(
                fields=["subject_identifier"], name="meta_ae_aei_subject_5e81f2_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="aeinitial",
            index=models.Index(
                fields=[
                    "action_identifier",
                    "action_item",
                    "related_action_item",
                    "parent_action_item",
                ],
                name="meta_ae_aei_action__8e0b23_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="aeinitial",
            index=models.Index(
                fields=["subject_identifier", "action_identifier", "site"],
                name="meta_ae_aei_subject_361c43_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="aesusar",
            index=models.Index(
                fields=["subject_identifier"], name="meta_ae_aes_subject_b8259b_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="aesusar",
            index=models.Index(
                fields=[
                    "action_identifier",
                    "action_item",
                    "related_action_item",
                    "parent_action_item",
                ],
                name="meta_ae_aes_action__ae027e_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="aetmg",
            index=models.Index(
                fields=["subject_identifier"], name="meta_ae_aet_subject_9a33b1_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="aetmg",
            index=models.Index(
                fields=[
                    "action_identifier",
                    "action_item",
                    "related_action_item",
                    "parent_action_item",
                ],
                name="meta_ae_aet_action__d28be4_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="deathreport",
            index=models.Index(
                fields=[
                    "action_identifier",
                    "action_item",
                    "related_action_item",
                    "parent_action_item",
                ],
                name="meta_ae_dea_action__4c8369_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="deathreporttmg",
            index=models.Index(
                fields=["subject_identifier"], name="meta_ae_dea_subject_910d26_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="deathreporttmg",
            index=models.Index(
                fields=[
                    "action_identifier",
                    "action_item",
                    "related_action_item",
                    "parent_action_item",
                ],
                name="meta_ae_dea_action__ea8160_idx",
            ),
        ),
    ]



import os.path
from pathlib import Path

from edc_sites.site import sites
from pylabels import Sheet, Specification
from reportlab.graphics.barcode.widgets import BarcodeStandard39
from reportlab.graphics.charts.textlabels import Label as RlLabel
from reportlab.graphics.shapes import String

from ..models import LabelData


class Label:
    """
    this should be based on a generated dataset using the stock transfer
    request data.

    maybe include a printout for the site to review the IN-STOCK vs ORDER
    status for each subject in a stock transfer request

    subject1 IN-STOCK
    subject2 ORDER
    subject3 ORDER
    """

    def __init__(self, obj: LabelData):
        self.label = obj
        self.gender = obj.gender
        self.site_id = obj.site_id
        self.subject_identifier = obj.subject_identifier
        self.reference = obj.reference
        self.sid = obj.sid
        self.site_name = sites.get(self.site_id).title


def draw_label(label, width, height, data: LabelData):
    data = Label(data)
    br = BarcodeStandard39(
        humanReadable=True, checksum=False, barHeight=30, barWidth=0.7, gap=1.7
    )
    br.value = data.reference
    br.x = width - 140
    br.y = 25
    label.add(br)
    label.add(String(15, height - 20, f"META III Study - {data.site_name}", fontSize=10))
    label.add(
        String(
            width - 120, height - 40, f"{data.subject_identifier}{data.gender}", fontSize=12
        )
    )
    label.add(String(15, height - 40, "Dawa kwa ajili ya utafiti", fontSize=10))
    label.add(String(15, height - 50, "wa META III.", fontSize=10))
    label.add(String(15, height - 70, "Meza vidonge vinne usiku tu.", fontSize=10))
    label.add(String(15, 20, "128 tabs", fontSize=10))
    lab = RlLabel(x=width - 20, y=40, fontSize=10, angle=90)
    # lab.boxAnchor = "ne"
    # lab.dx = 0
    # lab.dy = -20
    lab.setText(data.sid)
    label.add(lab)
    return label


def print_sheets(labels: list[Label], path: Path | None = None, verbose: bool | None = None):
    specs = Specification(
        210,
        297,
        2,
        6,
        96,
        42,
        corner_radius=0,
        top_margin=21,
        left_margin=8,
        right_margin=8,
        bottom_margin=22,
        left_padding=2,
        right_padding=2,
        top_padding=2,
        bottom_padding=2,
    )

    if not path:
        path = Path(os.path.dirname(__file__))
    path = str(path / "meta_labels.pdf")
    sheet = Sheet(specs, draw_label, border=True)

    sheet.add_labels(labels)

    # Save the file and we are done.
    sheet.save(path)
    msg = "{0:d} label(s) output on {1:d} page(s) to {2}.".format(
        sheet.label_count, sheet.page_count, path
    )
    if verbose:
        print(msg)
    return msg



from django.contrib.admin import AdminSite as DjangoAdminSite
from edc_dashboard.utils import get_bootstrap_version

from .apps import AppConfig


class AdminSite(DjangoAdminSite):
    app_index_template = "edc_model_admin/admin/app_index.html"
    login_template = f"edc_auth/bootstrap{get_bootstrap_version()}/login.html"
    logout_template = f"edc_auth/bootstrap{get_bootstrap_version()}/login.html"
    enable_nav_sidebar = False  # DJ 3.1
    final_catch_all_view = True  # DJ 3.2
    site_title = AppConfig.verbose_name

    # Text to put in each page's <h1>.
    site_header = AppConfig.verbose_name

    # Text to put at the top of the admin index page.
    index_title = "Site administration"
    site_url = "/administration/"

    def __init__(self, **kwargs):
        super().__init__(name="meta_edc")


meta_edc_admin = AdminSite()



from django import forms
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from edc_consent.form_validators import SubjectConsentFormValidatorMixin
from edc_form_validators import FormValidator


class SubjectConsentFormValidator(SubjectConsentFormValidatorMixin, FormValidator):
    def validate_identity(self) -> None:
        """Override to validate `identity_type` is a hospital
        number and `identity` matches the screening form.
        """
        if self.cleaned_data.get("identity_type") != "hospital_no":
            raise forms.ValidationError({"identity_type": _("Expected 'hospital number'.")})

        if self.subject_screening.hospital_identifier != (
            self.cleaned_data.get("identity") or getattr(self.instance, "identity", "")
        ):
            raise forms.ValidationError(
                {
                    "identity": (
                        _("The hospital identifier does not match that reported at screening.")
                    )
                }
            )

    def validate_consent_datetime(self):
        if getattr(settings, "EDC_CONSENT_BYPASS_CONSENT_DATETIME_VALIDATION", False):
            return None
        return super().validate_consent_datetime()



from datetime import datetime
from zoneinfo import ZoneInfo

import time_machine
from dateutil.relativedelta import relativedelta
from django.core.exceptions import ObjectDoesNotExist
from django.test import TestCase, tag
from edc_constants.constants import NO, YES
from edc_utils import get_utcnow
from model_bakery import baker

from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin
from meta_subject.models import SubjectVisit
from meta_visit_schedule.constants import (
    MONTH1,
    MONTH3,
    MONTH6,
    MONTH9,
    MONTH12,
    MONTH15,
    MONTH18,
    MONTH21,
    MONTH24,
    MONTH27,
    MONTH30,
    MONTH33,
    MONTH36,
    MONTH39,
    MONTH42,
    MONTH45,
    MONTH48,
    WEEK2,
)


@tag("mnsi")
@time_machine.travel(datetime(2022, 6, 11, 8, 00, tzinfo=ZoneInfo("UTC")))
class TestMnsiRequired(MetaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.baseline_datetime = get_utcnow() - relativedelta(months=6)

        six_months_ago = self.baseline_datetime
        self.subject_screening = self.get_subject_screening(
            report_datetime=six_months_ago,
            eligibility_datetime=six_months_ago,
        )
        self.subject_consent = self.get_subject_consent(
            self.subject_screening,
            consent_datetime=six_months_ago,
        )
        self.subject_visit = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
            appt_datetime=six_months_ago,
        )

    def get_visit(self, visit_code: str):
        """Returns a scheduled visit with the specified visit code.

        If the visit already exists, it will be returned. If not, it
        will be created (along with any interim visits) and returned.
        """
        subject_identifier = self.subject_consent.subject_identifier
        try:
            subject_visit = SubjectVisit.objects.get(
                subject_identifier=subject_identifier,
                visit_code=visit_code,
                visit_code_sequence=0,
            )
        except ObjectDoesNotExist:
            subject_visit = (
                SubjectVisit.objects.filter(
                    subject_identifier=subject_identifier,
                    visit_code_sequence=0,
                )
                .order_by("visit_code")
                .last()
            )
            while subject_visit.visit_code != visit_code:
                subject_visit = self.get_next_subject_visit(subject_visit)

        self.assertEqual(subject_visit.visit_code, visit_code)
        self.assertEqual(subject_visit.visit_code_sequence, 0)
        return subject_visit

    @staticmethod
    def set_mnsi_status(subject_visit, mnsi_performed):
        visit_mnsi = baker.make(
            "meta_subject.mnsi",
            subject_visit=subject_visit,
            mnsi_performed=mnsi_performed,
        )
        visit_mnsi.save()

    def test_mnsi_not_required_at_baseline(self):
        crfs = self.get_crf_metadata(self.subject_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_not_required_at_2w(self):
        crfs = self.get_crf_metadata(self.get_visit(visit_code=WEEK2))
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_required_at_1m(self):
        crfs = self.get_crf_metadata(self.get_visit(visit_code=MONTH1))
        self.assertIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_not_required_at_3m_if_already_performed_at_1m(self):
        # MNSI shouldn't be required at any point after it has been performed
        month1_visit = self.get_visit(visit_code=MONTH1)
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=YES)

        month3_visit = self.get_visit(visit_code=MONTH3)
        crfs = self.get_crf_metadata(month3_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_required_at_3m_if_not_performed_at_1m(self):
        month1_visit = self.get_visit(visit_code=MONTH1)
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=NO)

        month3_visit = self.get_visit(visit_code=MONTH3)
        crfs = self.get_crf_metadata(month3_visit)
        self.assertIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_not_required_at_6m_if_already_performed_at_1m(self):
        # MNSI shouldn't be required at any point after it has been performed
        month1_visit = self.get_visit(visit_code=MONTH1)
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=YES)

        month3_visit = self.get_visit(visit_code=MONTH3)
        crfs = self.get_crf_metadata(month3_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

        month6_visit = self.get_visit(visit_code=MONTH6)
        crfs = self.get_crf_metadata(month6_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_not_required_at_6m_if_already_performed_at_3m(self):
        month1_visit = self.get_visit(visit_code=MONTH1)
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=NO)

        # MNSI shouldn't be required at any point after it has been performed
        month3_visit = self.get_visit(visit_code=MONTH3)
        self.set_mnsi_status(subject_visit=month3_visit, mnsi_performed=YES)

        month6_visit = self.get_visit(visit_code=MONTH6)
        crfs = self.get_crf_metadata(month6_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_required_at_6m_if_not_performed_by_3m(self):
        month1_visit = self.get_visit(visit_code=MONTH1)
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=NO)

        month3_visit = self.get_visit(visit_code=MONTH3)
        self.set_mnsi_status(subject_visit=month3_visit, mnsi_performed=NO)

        month6_visit = self.get_visit(visit_code=MONTH6)
        crfs = self.get_crf_metadata(month6_visit)
        self.assertIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_not_required_at_9m_if_already_performed_at_1m(self):
        # MNSI shouldn't be required at any point after it has been performed
        month1_visit = self.get_visit(visit_code=MONTH1)
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=YES)

        month3_visit = self.get_visit(visit_code=MONTH3)
        crfs = self.get_crf_metadata(month3_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

        month6_visit = self.get_visit(visit_code=MONTH6)
        crfs = self.get_crf_metadata(month6_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

        month9_visit = self.get_visit(visit_code=MONTH9)
        crfs = self.get_crf_metadata(month9_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_not_required_at_9m_if_already_performed_at_3m(self):
        month1_visit = self.get_visit(visit_code=MONTH1)
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=NO)

        # MNSI shouldn't be required at any point after it has been performed
        month3_visit = self.get_visit(visit_code=MONTH3)
        self.set_mnsi_status(subject_visit=month3_visit, mnsi_performed=YES)

        month6_visit = self.get_visit(visit_code=MONTH6)
        crfs = self.get_crf_metadata(month6_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

        month9_visit = self.get_visit(visit_code=MONTH9)
        crfs = self.get_crf_metadata(month9_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_not_required_at_9m_if_already_performed_at_6m(self):
        month1_visit = self.get_visit(visit_code=MONTH1)
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=NO)

        month3_visit = self.get_visit(visit_code=MONTH3)
        self.set_mnsi_status(subject_visit=month3_visit, mnsi_performed=NO)

        # MNSI shouldn't be required at any point after it has been performed
        month6_visit = self.get_visit(visit_code=MONTH6)
        self.set_mnsi_status(subject_visit=month6_visit, mnsi_performed=YES)

        month9_visit = self.get_visit(visit_code=MONTH9)
        crfs = self.get_crf_metadata(month9_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_not_required_between_9m_and_33m_when_performed_in_first_6m(self):
        month1_visit = self.get_visit(visit_code=MONTH1)
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=NO)

        month3_visit = self.get_visit(visit_code=MONTH3)
        self.set_mnsi_status(subject_visit=month3_visit, mnsi_performed=YES)

        for visit_code in [
            MONTH9,
            MONTH12,
            MONTH15,
            MONTH18,
            MONTH21,
            MONTH24,
            MONTH27,
            MONTH30,
            MONTH33,
        ]:
            with self.subTest(visit_code=visit_code):
                subject_visit = self.get_visit(visit_code=visit_code)
                self.assertEqual(subject_visit.visit_code, visit_code)
                self.assertEqual(subject_visit.visit_code_sequence, 0)
                crfs = self.get_crf_metadata(subject_visit)
                self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_not_required_between_9m_and_33m_even_if_not_performed_first_6m(self):
        month1_visit = self.get_visit(visit_code=MONTH1)
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=NO)

        month3_visit = self.get_visit(visit_code=MONTH3)
        self.set_mnsi_status(subject_visit=month3_visit, mnsi_performed=NO)

        month6_visit = self.get_visit(visit_code=MONTH6)
        self.set_mnsi_status(subject_visit=month6_visit, mnsi_performed=NO)

        for visit_code in [
            MONTH9,
            MONTH12,
            MONTH15,
            MONTH18,
            MONTH21,
            MONTH24,
            MONTH27,
            MONTH30,
            MONTH33,
        ]:
            with self.subTest(visit_code=visit_code):
                subject_visit = self.get_visit(visit_code=visit_code)
                self.assertEqual(subject_visit.visit_code, visit_code)
                self.assertEqual(subject_visit.visit_code_sequence, 0)
                crfs = self.get_crf_metadata(subject_visit)
                self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_required_between_36m_and_45m(self):
        month1_visit = self.get_visit(visit_code=MONTH1)
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=YES)

        for visit_code in [MONTH36, MONTH39, MONTH42, MONTH45]:
            with self.subTest(visit_code=visit_code):
                subject_visit = self.get_visit(visit_code=visit_code)
                crfs = self.get_crf_metadata(subject_visit)
                self.assertIn("meta_subject.mnsi", [o.model for o in crfs.all()])
                self.set_mnsi_status(subject_visit=subject_visit, mnsi_performed=NO)

    def test_mnsi_only_required_once_between_36m_and_45m(self):
        month1_visit = self.get_visit(visit_code=MONTH1)
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=YES)

        month36_visit = self.get_visit(visit_code=MONTH36)
        crfs = self.get_crf_metadata(month36_visit)
        self.assertIn("meta_subject.mnsi", [o.model for o in crfs.all()])
        self.set_mnsi_status(subject_visit=month36_visit, mnsi_performed=NO)

        month39_visit = self.get_next_subject_visit(month36_visit)
        crfs = self.get_crf_metadata(month39_visit)
        self.assertIn("meta_subject.mnsi", [o.model for o in crfs.all()])
        self.set_mnsi_status(subject_visit=month39_visit, mnsi_performed=NO)

        month42_visit = self.get_next_subject_visit(month39_visit)
        crfs = self.get_crf_metadata(month42_visit)
        self.assertIn("meta_subject.mnsi", [o.model for o in crfs.all()])
        self.set_mnsi_status(subject_visit=month42_visit, mnsi_performed=YES)

        month45_visit = self.get_next_subject_visit(month42_visit)
        crfs = self.get_crf_metadata(month45_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

    def test_mnsi_required_at_48m(self):
        month1_visit = self.get_visit(visit_code=MONTH1)
        crfs = self.get_crf_metadata(month1_visit)
        self.assertIn("meta_subject.mnsi", [o.model for o in crfs.all()])
        self.set_mnsi_status(subject_visit=month1_visit, mnsi_performed=YES)

        month3_visit = self.get_next_subject_visit(month1_visit)
        crfs = self.get_crf_metadata(month3_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

        month36_visit = self.get_visit(visit_code=MONTH36)
        crfs = self.get_crf_metadata(month36_visit)
        self.assertIn("meta_subject.mnsi", [o.model for o in crfs.all()])
        self.set_mnsi_status(subject_visit=month36_visit, mnsi_performed=YES)

        month39_visit = self.get_next_subject_visit(month36_visit)
        crfs = self.get_crf_metadata(month39_visit)
        self.assertNotIn("meta_subject.mnsi", [o.model for o in crfs.all()])

        month48_visit = self.get_visit(visit_code=MONTH48)
        crfs = self.get_crf_metadata(month48_visit)
        self.assertIn("meta_subject.mnsi", [o.model for o in crfs.all()])



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_model_admin.list_filters import ReportDateListFilter
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import meta_pharmacy_admin
from ..forms import SubstitutionsForm
from ..models import Substitutions


@admin.register(Substitutions, site=meta_pharmacy_admin)
class SubstitutionsAdmin(
    SiteModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    SimpleHistoryAdmin,
):
    """Admin class for meta_pharmacy.Substitutions"""

    form = SubstitutionsForm

    autocomplete_fields = ["rx"]

    add_instructions = "This form is used to record any substitions made for IMP refills"

    fieldsets = (
        (
            None,
            {"fields": ("report_datetime",)},
        ),
        ("Original label on bottle", {"fields": ("sid", "visit_no")}),
        (
            "Substitution / Changed label",
            {
                "description": "The label on this bottle has been changed to the following",
                "fields": ("dispensed_sid", "updated_visit_no"),
            },
        ),
        (
            "Subject",
            {
                "fields": ("rx",),
            },
        ),
        audit_fieldset_tuple,
    )

    list_display = (
        "sid",
        "dispensed_sid",
        "arm_match",
        "subject_identifier",
        "report_date",
    )

    radio_fields = {"arm_match": admin.VERTICAL}

    list_filter = ("arm_match", ReportDateListFilter)

    search_fields = ["subject_identifier", "sid", "dispensed_sid"]

    @admin.display(description="Report date", ordering="report_datetime")
    def report_date(self, obj) -> str | None:
        if obj.report_datetime:
            return obj.report_datetime.date()
        return None



from django.db import models
from edc_qareports.model_mixins import QaReportModelMixin, qa_reports_permissions


class Endpoints(QaReportModelMixin, models.Model):
    """A QA report model updated using an admin
    action or manually.
    """

    visit_code = models.IntegerField(null=True)

    fasting = models.CharField(max_length=10, null=True)

    fbg_date = models.DateField(null=True)

    fbg_value = models.FloatField(null=True)

    ogtt_value = models.FloatField(null=True)

    endpoint_label = models.CharField(max_length=250, null=True)

    baseline_date = models.DateField(null=True)

    offstudy_date = models.DateField(null=True)

    offstudy_reason = models.CharField(max_length=250, null=True)

    class Meta(QaReportModelMixin.Meta):
        verbose_name = "Endpoints (DM)"
        verbose_name_plural = "Endpoints (DM)"
        default_permissions = qa_reports_permissions



# Generated by Django 3.2.11 on 2022-01-28 14:19

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("meta_screening", "0029_auto_20211123_1645"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicalscreeningpartone",
            name="diet_and_lifestyle",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningpartone",
            name="diet_and_lifestyle_duration",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningpartthree",
            name="diet_and_lifestyle",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningpartthree",
            name="diet_and_lifestyle_duration",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningparttwo",
            name="diet_and_lifestyle",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningparttwo",
            name="diet_and_lifestyle_duration",
        ),
        migrations.RemoveField(
            model_name="historicalsubjectscreening",
            name="diet_and_lifestyle",
        ),
        migrations.RemoveField(
            model_name="historicalsubjectscreening",
            name="diet_and_lifestyle_duration",
        ),
        migrations.RemoveField(
            model_name="subjectscreening",
            name="diet_and_lifestyle",
        ),
        migrations.RemoveField(
            model_name="subjectscreening",
            name="diet_and_lifestyle_duration",
        ),
    ]



from edc_sites.site import sites as site_sites

list_data = {
    "edc_pharmacy.location": [
        ("central", "Central"),
        *[(ss.name, ss.description) for ss in site_sites.all(aslist=True)],
    ],
}



# Generated by Django 5.1.2 on 2024-10-16 00:53

import django.db.models.deletion
import edc_utils.date
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0051_remove_endpoints_baseline_datetime_and_more"),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.CreateModel(
            name="LastImpVisit",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("report_model", models.CharField(max_length=50)),
                ("subject_identifier", models.CharField(max_length=25)),
                ("created", models.DateTimeField(default=edc_utils.date.get_utcnow)),
                ("reference_date", models.DateField(null=True)),
                ("imp_visit_code", models.FloatField(null=True)),
                ("imp_visit_date", models.DateField(null=True)),
                ("next_visit_code", models.FloatField(null=True)),
                ("next_appt_date", models.DateField(null=True)),
                ("visit_code", models.CharField(max_length=15, null=True)),
                ("visit_code_sequence", models.IntegerField(null=True)),
                ("days_since", models.IntegerField(null=True)),
                ("days_until", models.IntegerField(null=True)),
                (
                    "site",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.DO_NOTHING, to="sites.site"
                    ),
                ),
            ],
            options={
                "verbose_name": "Last IMP Visit",
                "verbose_name_plural": "Last IMP Visits",
                "abstract": False,
                "default_permissions": ("view", "export", "viewallsites"),
                "indexes": [
                    models.Index(
                        fields=["subject_identifier", "site"],
                        name="meta_report_subject_4a9e15_idx",
                    )
                ],
            },
        ),
    ]



# Generated by Django 4.2.11 on 2024-05-16 00:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "meta_screening",
            "0063_alter_historicalscreeningpartone_fasting_duration_str_and_more",
        ),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicalscreeningpartone",
            name="fasting_duration_minutes",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningpartone",
            name="repeat_fasting_duration_minutes",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningpartthree",
            name="fasting_duration_minutes",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningpartthree",
            name="repeat_fasting_duration_minutes",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningparttwo",
            name="fasting_duration_minutes",
        ),
        migrations.RemoveField(
            model_name="historicalscreeningparttwo",
            name="repeat_fasting_duration_minutes",
        ),
        migrations.RemoveField(
            model_name="historicalsubjectscreening",
            name="fasting_duration_minutes",
        ),
        migrations.RemoveField(
            model_name="historicalsubjectscreening",
            name="repeat_fasting_duration_minutes",
        ),
        migrations.RemoveField(
            model_name="subjectscreening",
            name="fasting_duration_minutes",
        ),
        migrations.RemoveField(
            model_name="subjectscreening",
            name="repeat_fasting_duration_minutes",
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="fasting_duration_delta",
            field=models.DurationField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="repeat_fasting_duration_delta",
            field=models.DurationField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="fasting_duration_delta",
            field=models.DurationField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="repeat_fasting_duration_delta",
            field=models.DurationField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="fasting_duration_delta",
            field=models.DurationField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="repeat_fasting_duration_delta",
            field=models.DurationField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="fasting_duration_delta",
            field=models.DurationField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="repeat_fasting_duration_delta",
            field=models.DurationField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="fasting_duration_delta",
            field=models.DurationField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="repeat_fasting_duration_delta",
            field=models.DurationField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
    ]



# Generated by Django 5.1 on 2024-10-07 22:05

import _socket
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_pharmacy", "0005_auto_20240911_0352"),
    ]

    operations = [
        migrations.CreateModel(
            name="LotNumber",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                        verbose_name="Hostname created",
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                        verbose_name="Hostname modified",
                    ),
                ),
                (
                    "device_created",
                    models.CharField(blank=True, max_length=10, verbose_name="Device created"),
                ),
                (
                    "device_modified",
                    models.CharField(
                        blank=True, max_length=10, verbose_name="Device modified"
                    ),
                ),
                (
                    "locale_created",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale created",
                    ),
                ),
                (
                    "locale_modified",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale modified",
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("lot_no", models.CharField(max_length=25, unique=True)),
                (
                    "allocation",
                    models.CharField(
                        choices=[("active", "Active"), ("placebo", "Placebo")], max_length=25
                    ),
                ),
                ("expiration_date", models.DateField()),
                ("qty", models.IntegerField(null=True)),
            ],
            options={
                "verbose_name": "Lot Number",
                "verbose_name_plural": "Lot Numbers",
                "abstract": False,
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "default_manager_name": "objects",
                "indexes": [
                    models.Index(
                        fields=["modified", "created"], name="meta_pharma_modifie_a98e60_idx"
                    ),
                    models.Index(
                        fields=["user_modified", "user_created"],
                        name="meta_pharma_user_mo_1be860_idx",
                    ),
                ],
            },
        ),
        migrations.CreateModel(
            name="Label",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                        verbose_name="Hostname created",
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                        verbose_name="Hostname modified",
                    ),
                ),
                (
                    "device_created",
                    models.CharField(blank=True, max_length=10, verbose_name="Device created"),
                ),
                (
                    "device_modified",
                    models.CharField(
                        blank=True, max_length=10, verbose_name="Device modified"
                    ),
                ),
                (
                    "locale_created",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale created",
                    ),
                ),
                (
                    "locale_modified",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale modified",
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("rx_label_reference", models.CharField(max_length=15, unique=True)),
                ("printed_datetime", models.DateTimeField(null=True)),
                ("printed", models.BooleanField(default=False)),
                ("scanned", models.BooleanField(default=False)),
                ("scanned_datetime", models.DateTimeField(null=True)),
                (
                    "rx",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT, to="meta_pharmacy.rx"
                    ),
                ),
                (
                    "lot_no",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="meta_pharmacy.lotnumber",
                    ),
                ),
            ],
            options={
                "verbose_name": "Label",
                "verbose_name_plural": "Labels",
                "abstract": False,
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "default_manager_name": "objects",
                "indexes": [
                    models.Index(
                        fields=["modified", "created"], name="meta_pharma_modifie_ebebcc_idx"
                    ),
                    models.Index(
                        fields=["user_modified", "user_created"],
                        name="meta_pharma_user_mo_cd7246_idx",
                    ),
                ],
            },
        ),
    ]



# Generated by Django 3.2.13 on 2022-07-04 15:41

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0120_alter_birthoutcomes_managers_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicalbirthoutcomes",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Birth Outcomes",
                "verbose_name_plural": "historical Birth Outcomes",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalbloodresultsfbc",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Blood Result: FBC",
                "verbose_name_plural": "historical Blood Results: FBC",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalbloodresultsglu",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Blood Result: Glucose",
                "verbose_name_plural": "historical Blood Results: Glucose",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalbloodresultshba1c",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Blood Result: HbA1c",
                "verbose_name_plural": "historical Blood Results: HbA1c",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalbloodresultsins",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Blood Result: Insulin",
                "verbose_name_plural": "historical Blood Results: Insulin",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalbloodresultslft",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Blood Result: LFT",
                "verbose_name_plural": "historical Blood Results: LFT",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalbloodresultslipid",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Blood Result: Lipids",
                "verbose_name_plural": "historical Blood Results: Lipids",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalbloodresultsrft",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Blood Result: RFT",
                "verbose_name_plural": "historical Blood Results: RFT",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalcomplications",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Presence of Complications",
                "verbose_name_plural": "historical Presence of Complications",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalcomplicationsglycemia",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Complications (Raised Glycemia)",
                "verbose_name_plural": "historical Complications (Raised Glycemia)",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalconcomitantmedication",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Concomitant Medication",
                "verbose_name_plural": "historical Concomitant Medication",
            },
        ),
        migrations.AlterModelOptions(
            name="historicaldelivery",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Delivery",
                "verbose_name_plural": "historical Delivery",
            },
        ),
        migrations.AlterModelOptions(
            name="historicaldietandlifestyle",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Clinic follow up: Examination",
                "verbose_name_plural": "historical Clinic follow up: Examination",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalegfrnotification",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical eGFR Notification",
                "verbose_name_plural": "historical eGFR Notifications",
            },
        ),
        migrations.AlterModelOptions(
            name="historicaleq5d3l",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical EuroQol EQ-5D-3L Instrument",
                "verbose_name_plural": "historical EuroQol EQ-5D-3L Instrument",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalfollowupexamination",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Clinic follow up: Examination",
                "verbose_name_plural": "historical Clinic follow up: Examination",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalfollowupvitals",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Clinic follow up: Vitals",
                "verbose_name_plural": "historical Clinic follow ups: Vitals",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalglucose",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Glucose (IFG, OGTT)",
                "verbose_name_plural": "historical Glucose (IFG, OGTT)",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalhealtheconomics",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Health Economics (M2)",
                "verbose_name_plural": "historical Health Economics (M2)",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalhealtheconomicssimple",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Health Economics (M3)",
                "verbose_name_plural": "historical Health Economics (M3)",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalhepatitistest",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Hepatitis Tests",
                "verbose_name_plural": "historical Hepatitis Tests",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalmalariatest",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Malaria Test",
                "verbose_name_plural": "historical Malaria Tests",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalmedicationadherence",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Medication Adherence",
                "verbose_name_plural": "historical Medication Adherence",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalmnsi",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Michigan Neuropathy Screening Instrument (MNSI)",
                "verbose_name_plural": "historical Michigan Neuropathy Screening Instrument (MNSI)",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalotherarvregimens",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Other ARV Regimens",
                "verbose_name_plural": "historical Other ARV Regimens",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalotherarvregimensdetail",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Other ARV Regimens Detail",
                "verbose_name_plural": "historical Other ARV Regimens Detail",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalpatienthistory",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Patient History",
                "verbose_name_plural": "historical Patient History",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalphysicalexam",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Physical Exam",
                "verbose_name_plural": "historical Physical Exams",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalpregnancyupdate",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Pregnancy Update",
                "verbose_name_plural": "historical Pregnancy Update",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalsf12",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical SF-12 Health Survey",
                "verbose_name_plural": "historical SF-12 Health Survey",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalstudymedication",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Study Medication",
                "verbose_name_plural": "historical Study Medication",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalsubjectrequisition",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical subject requisition",
                "verbose_name_plural": "historical subject requisitions",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalsubjectvisit",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical subject visit",
                "verbose_name_plural": "historical subject visits",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalsubjectvisitmissed",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Missed Visit Report",
                "verbose_name_plural": "historical Missed Visit Report",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalurinedipsticktest",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Urine Dipstick Test",
                "verbose_name_plural": "historical Urine Dipstick Tests",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalurinepregnancy",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Urine Pregnancy",
                "verbose_name_plural": "historical Urine Pregnancys",
            },
        ),
        migrations.AddField(
            model_name="historicalmedicationadherence",
            name="pill_count_not_performed_reason",
            field=models.TextField(
                blank=True, null=True, verbose_name="Was a pill count performed"
            ),
        ),
        migrations.AddField(
            model_name="medicationadherence",
            name="pill_count_not_performed_reason",
            field=models.TextField(
                blank=True, null=True, verbose_name="Was a pill count performed"
            ),
        ),
        migrations.AlterField(
            model_name="historicalbirthoutcomes",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsfbc",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsglu",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultshba1c",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsins",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslipid",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsrft",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalcomplications",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalcomplicationsglycemia",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalconcomitantmedication",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicaldelivery",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicaldietandlifestyle",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalegfrnotification",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicaleq5d3l",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalfollowupexamination",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalfollowupvitals",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalglucose",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomics",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicssimple",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalhepatitistest",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalmalariatest",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalmedicationadherence",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalmnsi",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalotherarvregimens",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalotherarvregimensdetail",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalpatienthistory",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalphysicalexam",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalpregnancyupdate",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalsf12",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalstudymedication",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalsubjectrequisition",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalsubjectvisit",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalsubjectvisitmissed",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalurinedipsticktest",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalurinepregnancy",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
    ]
